@@ -34,7 +34,9 @@ from seamm.join_node import Join  # noqa: F401
 from seamm.tk_join_node import TkJoin  # noqa: F401
 from seamm.builtins import JoinStep  # noqa: F401
 from seamm.tk_job_handler import TkJobHandler  # noqa: F401
+from .dashboard_handler import DashboardHandler  # noqa: F401
 from seamm.run_flowchart import run as run_flowchart  # noqa: F401
+from .seammrc import SEAMMrc  # noqa: F401
 
 wrap_text = textwrap.TextWrapper(width=120)
 wrap_stdout = textwrap.TextWrapper(width=120)
